function SR = mapsrColor(LRImages, model, interChannelWeight, varargin)
    
    if nargin > 3
        solverParams = varargin{end};
    else
        % Use default solver parameters
        solverParams = SRSolverParams;
    end
    
    if ~iscell(LRImages)
        numChannels = length(LRImages);
        for channelIdx = 1:numChannels
            y{channelIdx} = squeeze(LRImages(:,:,channelIdx,:));
        end
        LRImages = y;
    end
        
    % Assemble equation system to be solved consisting of low-resolution
    % observations and the system matrix.
    if solverParams.verbose
        disp('Assemble equation system (system matrix)...');
    end
    [W, ~] = composeSREquationSystem(LRImages{1}, model);
    Wt = W';
    numChannels = length(LRImages);
    for channelIdx = 1:numChannels
        LR{channelIdx} = imageToVector(LRImages{channelIdx});
    end
        
    if ~isfield(model, 'SR') || isempty(model.SR)
        % Compute initial guess for the super-resolved image. We use the
        % "average image" estimated from the system matrix and photometric
        % parameters (if available).
        if solverParams.verbose
            disp('Compute average image used as initial guess...');
        end
        for channelIdx = 1:numChannels
            SR{channelIdx} = getInitialSRImage(W, LR{channelIdx}, model.photometricParams);
        end
    else
        % Use initial guess provided by the user and reshape 2-D image into
        % parameter vector.
        for channelIdx = 1:length(model.SR)
            SR{channelIdx} = imageToVector(model.SR{channelIdx});
        end
    end
    
    scgOptions = setupSCGOptions(solverParams);
    if solverParams.verbose
        disp('Minimize objective function...');
    end 
    iter = 1;
    isConverged = false;
    while iter <= solverParams.maxIter && ~isConverged
        % Cyclic coordinate descent optimization for different channels
        for channelIdx = 1:numChannels
            SR_new{channelIdx} = scg(@mapfunc, SR{channelIdx}', scgOptions, @mapfunc_grad, model, LR{channelIdx}, W, Wt, SR, channelIdx, interChannelWeight, model.magFactor * size(LRImages{1}(:,:,1)));
            SR_new{channelIdx} = SR_new{channelIdx}';     
        end
        
        for channelIdx = 1:numChannels
            if max( abs(SR_new{channelIdx}(:) - SR{channelIdx}(:)) ) < solverParams.tolX
                isConverged(channelIdx) = true;
            end
        end
        isConverged = all(isConverged);
        
        % Update super-resolved image for next iteration.
        SR = SR_new;
        iter = iter + 1;
    end
    
    % Reshape parameter vector to a 2D image.
    for channelIdx = 1:length(LRImages)
        x(:,:,channelIdx) = vectorToImage(SR{channelIdx}, model.magFactor * size(LRImages{1}(:,:,1)));
    end
    SR = x;
    if solverParams.verbose
        disp('DONE!');
    end
    
function scgOptions = setupSCGOptions(solverParams)

    scgOptions = zeros(1,18); 
    scgOptions(1) = solverParams.verbose;        
    scgOptions(2) = solverParams.tolX;   
    scgOptions(3) = solverParams.tolF;
    scgOptions(9) = solverParams.gradCheck;
    scgOptions(10) = solverParams.maxFunEvals;     
    scgOptions(14) = solverParams.maxIter;
    
function f = mapfunc(SR, model, LR, W, ~, SRchannels, channelIdx, interChannelWeight, imsize)
        
    if ~iscolumn(SR)
        % Reshape to column vector. 
        SR = SR';
    end
    
    % Evaluate the data fidelity term.
    fDataTerm = mapDataTerm(SR, model, LR, W);
    
    % Evaluate image prior for regularization the super-resolved estimate.
    fIntra = model.imagePrior.function(SR, model.imagePrior.parameters{1:end});
    
    fInter = 0;
    for c = 1:length(SRchannels)
        if c ~= channelIdx
            fInter = fInter + interChannelPrior(SRchannels{c}, SR, imsize);
        end
    end
    
    % Calculate objective function.
    f = fDataTerm + model.imagePrior.weight * fIntra + interChannelWeight * fInter;
                
function grad = mapfunc_grad(SR, model, LR, W, Wt, SRchannels, channelIdx, interChannelWeight, imsize)
        
    if ~iscolumn(SR)
        % Reshape to column vector. 
        SR = SR';
    end
    
    % Calculate gradient of the data fidelity term w.r.t. the
    % super-resolved image.
    gDataTerm = mapDataTerm_gradImage(SR, model, LR, W, Wt);
    
    % Calculate gradient of the regularization term w.r.t. the 
    % super-resolved image.
    gIntra = model.imagePrior.gradient(SR, model.imagePrior.parameters{1:end});
    
    gInter = 0;
    for c = 1:length(SRchannels)
        if c ~= channelIdx
            gInter = gInter + interChannelPrior_grad(SRchannels{c}, SR, imsize);
        end
    end
    
    % Sum up to total gradient
    grad = gDataTerm + model.imagePrior.weight * gIntra + interChannelWeight * gInter;
    grad = grad';
    
function f = interChannelPrior(xb, xg, imsize)
    
    % Reshape SR vector to image for further processing.
    Xb = vectorToImage(xb, imsize);
    Xg = vectorToImage(xg, imsize);
    
    % Pad image at the border to perform shift operations.
    P = 1;
    XbPad = padarray(Xb, [P P], 'symmetric');
    XgPad = padarray(Xg, [P P], 'symmetric');
    
    % Consider shifts in the interval [-P, +P].
    f = 0;
    for l=-P:P
        for m=-P:P
            % Shift by l and m pixels.
            Xb_shift = XbPad((1+P-l):(end-P-l), (1+P-m):(end-P-m));
            Xg_shift = XgPad((1+P-l):(end-P-l), (1+P-m):(end-P-m));
            f = f + sum(sum( (Xb_shift .* Xg - Xb .* Xg_shift).^2 ));
        end
    end
    
function g = interChannelPrior_grad(xb, xg, imsize)
    
    % Reshape SR vector to image for further processing.
    Xb = vectorToImage(xb, imsize);
    Xg = vectorToImage(xg, imsize);
    
    % Pad image at the border to perform shift operations.
    P = 1;
    XbPad = padarray(Xb, [P P], 'symmetric');
    XgPad = padarray(Xg, [P P], 'symmetric');
    
    % Consider shifts in the interval [-P, +P].
    G = zeros(size(Xb));
    for l=-P:P
        for m=-P:P
            % Shift by l and m pixels.
            Xb_shift = XbPad((1+P-l):(end-P-l), (1+P-m):(end-P-m));
            Xg_shift = XgPad((1+P-l):(end-P-l), (1+P-m):(end-P-m));
             
            L = Xb_shift .* Xg - Xb .* Xg_shift;
            Lpad = padarray(L, [P P], 0);
            Linv = Lpad((1+P+l):(end-P+l), (1+P+m):(end-P+m));
            Gk = 2 * (Xb_shift .* L - Xb .* Linv);
            
            G = G + Gk;            
        end
    end
    
    % Linearize gradient to a vector.
    g = imageToVector(G);
    