% Get matrix of the linear system for LM optimization.
function [A, b, priorConfidenceWeights, observationConfidenceWeights] = assembleEquationSystem(x, optStruct, model, calmsrParams, mu, priorConfidenceWeights, observationConfidenceWeights)

    LRImages = optStruct.LRImages;
    numFrames = optStruct.numFrames;
    M = optStruct.Size(1);
    N = optStruct.Size(2);
    
    % Get system matrix and Jacobian.
    [W_all, J] = getSystemMatrixAndJacobian(x, optStruct, model.motionParams, model.magFactor);
    m = M / model.magFactor;
    n = N / model.magFactor;
    [row, ~] = size(W_all);
    W1 = W_all(1:m*n,:);
    W = W_all(m*n+1:row,:);
    
    % Determine regularization matrix and prior weights.
    [T, priorConfidenceWeights] =  getRegularizationMatrix(calmsrParams.imagePrior, x, M, N, priorConfidenceWeights, calmsrParams);
    
    % Determine observation confidence weights.
    if ~isempty(calmsrParams.observationWeightingFunction)
        observationConfidenceWeights = calmsrParams.observationWeightingFunction.function( vec2cell(LRImages(:) - W_all * x, m*n), vec2cell(observationConfidenceWeights, m*n), calmsrParams.observationWeightingFunction.parameters{1:end});
        observationConfidenceWeights = cell2vec(observationConfidenceWeights);
    else
       observationConfidenceWeights = ones(numel(LRImages), 1); 
    end
    B = createSparse( sqrt(observationConfidenceWeights(m*n+1:end)) );
    B1 = createSparse( sqrt(observationConfidenceWeights(1:(m*n))) );
    
    % Compose the linear equation system A*x = b, where:
    % --------------------------------------------------
    %   A = [ A1 A2;
    %         A3 A4 ]
    %     = [ J'J + R^2    J'W;
    %         W'J          W1'W1 + W'W + deltaT ]
    % --------------------------------------------------
    A1 = J'*B*J;
    R = sqrt(calmsrParams.motionRegularizationWeight) * speye(size(A1,1) , size(A1,1));
    A1 = A1 + R^2;  
    % A2 
    A2 = J'*B*W;
    A12= cat(2, A1, A2);
    % A3
    A3 = W'*B*J;
    % A4
    A4 = W1'*B1*W1 + W'*B*W + calmsrParams.imageRegularizationWeight * T;
    A34 = cat(2, A3, A4);
    % Joint matrix.
    A = cat(1, A12, A34);
    S = T * x;

    if mu > 0
        % Compute matrix for LM iterations.
        A = A + mu * diag(diag(A));
    end
    
    % Determine residual error.
    g1 = LRImages(:,:,1); 
    g1 = g1(:);
    g = LRImages(:,:,2:numFrames); 
    g = g(:);
    r1 = g1 - W1 * x;
    r = g - W * x;
    
    % Right-hand side of the the linear equation system.
    % --------------------------------------------------
    %   b = [ J' * weights.* r
    %         W1'r1 + Wr - alphaTf ]
    % -------------------------------------------------- 
    b1 = J' * (B*r);
    b2 = W1' * (B1*r1) + W'* (B*r) - calmsrParams.imageRegularizationWeight*S;
    b = cat(1, b1, b2);
      
end 